from typing import Any, Dict, Optional
from urllib.parse import urljoin

import requests

import activesoup.csv
import activesoup.html
import activesoup.json_response
import activesoup.response
from activesoup.protocols import ActiveSoupResult, Resolver


class DriverError(RuntimeError):
    pass


class ContentResolver:
    def __init__(self):
        self._resolvers: Dict[str, Resolver] = {}

    def register(self, content_type: str, resolver: Resolver) -> None:
        self._resolvers[content_type] = resolver

    def resolve(self, response: requests.Response) -> ActiveSoupResult:
        content_type = response.headers.get("Content-Type", None)
        if content_type is not None:
            for k, v in self._resolvers.items():
                if content_type.startswith(k):
                    return v.resolve(response)

        return activesoup.response.Response(response, content_type)


class Driver:
    def __init__(self) -> None:
        self.session = requests.Session()
        self.last_response: Optional[ActiveSoupResult] = None
        self._raw_response: Optional[requests.Response] = None
        self.content_resolver = ContentResolver()
        self.content_resolver.register("text/html", activesoup.html.Resolver(self))
        self.content_resolver.register("text/csv", activesoup.csv.Resolver())
        self.content_resolver.register(
            "application/json", activesoup.json_response.Resolver()
        )

    def __enter__(self) -> "Driver":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.session.close()

    def resolve_url(self, possibly_relative_url) -> str:
        current_url_str = self.url
        if not current_url_str:
            return possibly_relative_url

        return urljoin(current_url_str, possibly_relative_url)

    def get(self, url, **kwargs):
        return self.do(requests.Request(method="GET", url=url))

    def do(self, request: requests.Request) -> "Driver":
        request.url = self.resolve_url(request.url)
        prepped = self.session.prepare_request(request)
        return self.handle_response(self.session.send(prepped))

    def handle_response(self, response: requests.Response) -> "Driver":
        if response.status_code in range(300, 304):
            redirected_to = response.headers.get("Location", None)
            if not redirected_to:
                raise DriverError("Found a redirect, but no onward location given")
            return self.get(redirected_to)

        self.last_response = self.content_resolver.resolve(response)
        self._raw_response = response

        return self

    @property
    def url(self) -> Optional[str]:
        return self.last_response.url if self.last_response is not None else None

    def __getattr__(self, item) -> Any:
        if not self.last_response:
            raise DriverError("Not on a page")

        return getattr(self.last_response, item)

    def __getitem__(self, item) -> Any:
        if not self.last_response:
            raise DriverError("Not on a page")

        return self.last_response[item]

    def __str__(self) -> str:
        last_resp_str = str(self.last_response) if self.last_response else "unbound"
        return f"<activesoup.driver.Driver[{last_resp_str}]>"



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['activesoup']

package_data = \
{'': ['*']}

install_requires = \
['html5lib>=0.9', 'requests>=2.9.0,<3.0.0']

setup_kwargs = {
    'name': 'activesoup',
    'version': '0.2.0',
    'description': 'A pure-python headless browser',
    'long_description': 'activesoup\n==========\n\n.. image:: https://github.com/jelford/activesoup/workflows/Build/badge.svg\n    :target: https://github.com/jelford/activesoup/actions?query=workflow%3Abuild\n\n.. image:: https://img.shields.io/pypi/v/activesoup.svg?maxAge=3600\n    :target: https://pypi.python.org/pypi?:action=display&name=activesoup\n\nA simple library for interacting with the web from python\n\nDescription\n-----------\n\n``activesoup`` combines familiar python web capabilities for convenient\nheadless "browsing" functionality:\n\n* Modern HTTP support with `requests <http://www.python-requests.org/>`__ -\n  connection pooling, sessions, ...\n* Convenient access to the web page with an interface inspired by\n  `beautifulsoup <https://www.crummy.com/software/BeautifulSoup/>`__ -\n  convenient HTML navigation.\n* Robust HTML parsing with\n  `html5lib <https://html5lib.readthedocs.org/en/latest/>`__ - parse the web\n  like browsers do.\n\nUse cases\n---------\n\nConsider using ``activesoup`` when:\n\n* You\'ve already checked out the `requests-html <https://github.com/kennethreitz/requests-html>`__\n* You need to actively interact with some web-page from Python (e.g. submitting\n  forms, downloading files)\n* You don\'t control the site you need to interact with (if you do, just make an\n  API).\n* You don\'t need javascript support (you\'ll need\n  `selenium <http://www.seleniumhq.org/projects/webdriver/>`__ or\n  `phantomjs <http://phantomjs.org/>`__).\n\nUsage examples\n--------------\n\nLog into a website, and download a CSV file that\'s access-protected:\n\n.. code-block:: python\n\n    >>> import activesoup\n\n    >>> # Start a session\n    >>> d = activesoup.Driver()\n\n    >>> page = d.get("https://httpbin.org/forms/post")\n\n    >>> # conveniently access elements, inspired by BeautifulSoup\n    >>> form = page.form\n\n    >>> # get the power of raw xpath search too\n    >>> form.find(\'.//input[@name="size"]\')\n    BoundTag<input>\n\n    >>> # inspect element attributes\n    >>> print([i[\'name\'] for i in form.find_all(\'input\')])\n    [\'custname\', \'custtel\', \'custemail\', \'size\', \'size\', \'size\', \'topping\', \'topping\', \'topping\', \'topping\', \'delivery\']\n\n    >>> # work actively with objects on the page\n    >>> r = form.submit({"custname": "john", "size": "small"})\n\n    >>> # responses parsed and ready based on content type\n    >>> r.keys()\n    dict_keys([\'args\', \'data\', \'files\', \'form\', \'headers\', \'json\', \'origin\', \'url\'])\n    >>> r[\'form\']\n    {\'custname\': \'john\', \'size\': \'small\', \'topping\': \'mushroom\'}\n\n    >>> # access the underlying requests.Session too\n    >>> d.session\n    <requests.sessions.Session object at 0x7f283dc95700>\n\n    >>> # log in with cookie support\n    >>> d.get(\'https://httpbin.org/cookies/set/foo/bar)\n    >>> d.session.cookies[\'foo\']\n    \'bar\'\n',
    'author': 'James Elford',
    'author_email': 'james.p.elford@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jelford/activesoup',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Union

import requests


class JsonResponse:
    def __init__(self, raw_response: requests.Response) -> None:
        self.raw_response = raw_response
        self.json = raw_response.json()

    def __getitem__(
        self, name: Union[str, int]
    ) -> Union[str, int, Dict[str, Any], List[Any]]:
        return self.json[name]

    @property
    def url(self) -> str:
        return self.raw_response.url

    def __getattr__(self, attr: str) -> Any:
        return getattr(self.json, attr)

    def __repr__(self) -> str:
        return "JsonResponse"

    def __str__(self) -> str:
        return "<[json]>"


class Resolver:
    def resolve(self, raw_response: requests.Response) -> JsonResponse:
        return JsonResponse(raw_response)



class UnknownResponseType(RuntimeError):
    pass


class Response:
    def __init__(self, raw_response, content_type):
        self._raw_response = raw_response
        self._content_type = content_type

    @property
    def url(self):
        return self._raw_response.url

    @property
    def status_code(self):
        return self._raw_response.status_code

    @property
    def response(self):
        return self._raw_response

    def __getattr__(self, attr):
        raise UnknownResponseType(
            f"Wasn't sure how to parse this response (type: {self._content_type}), can't dive any deeper"
        )
